/* agrandit.c - Agrandit une image
 * ____________________________________________________________________
 * Agrandit une image d'un coefficient entier passe en parametre
 * Le coefficient sera superieur ou egal a 2.
 * ____________________________________________________________________
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "io.h"
#include "effets/agrandit.h"

void usage(char* telNom)
{
	fprintf(stderr,"usage: %s [coeff] [image_source]\n",telNom);
	fprintf(stderr,"\tTout coeff inferieur a 2 sera ");
	fprintf(stderr,"assimile a 2\n");	
}

int main(int argc, char** argv)
{
	int leCoef; /* Coef d'agrandissement */
	struct image lImageSource;
	struct image lImageDest;
	char* leFichierDest;
	int leRetour;
	
	if(argc!=3)
	{
		usage(argv[0]);
		return ERROR_ARGS;
	}
	
	/* Tout coefficient inferieur a 2 vaut 2 */
	leCoef=isdigit(*argv[1])?atoi(argv[1]):-1;
	
	if(leCoef==-1)
	{
		fprintf(stderr,"Le coefficient est invalide.\n");
		usage(argv[0]);
		return ERROR_ARGS;	
	}
	
	if(leCoef<2)
	{
		leCoef=2;
	}
	
	leRetour=get_image(&lImageSource, argv[2]);
	if(leRetour==NO_ERROR)
	{	
		/* Generation du nom de fichier de destination */
		leFichierDest=get_file_name(argv[2],"_ZOOM.ppm");
	
		/* Agrandissement */
		agrandit(&lImageDest, &lImageSource, leCoef);

		/* Creation de l'image finale */
		leRetour=create_image(&lImageDest, leFichierDest);

		/* Desallocation de la memoire */
		ppm_free((void*)lImageSource.sonContenu);
		ppm_free((void*)lImageDest.sonContenu);
		ppm_free((void*)leFichierDest);
	}
	
	return leRetour;
}

/* Agrandit l'image source du coefficient passee en parametre
 * et ecrit le resultat de le fichier de destination
 */
void agrandit(struct image* telleImageDest,
	      const struct image* telleImageSource,
	      int telCoef)
{
	int laTailleImage;
	int i;
	
	/* Recopie des entetes de l'image source et creation d'une image de
	 * destination du double en largeur et en hauteur de l'image source.
	 */
	laTailleImage=telleImageSource->saLargeur*telleImageSource->saHauteur;
	copie_image_entete_px(telleImageDest,
			      telleImageSource,
			      telleImageSource->saLargeur*telCoef,
			      telleImageSource->saHauteur*telCoef);

	/* Parcours et agrandisssement de l'image source */
	for(i=0; i<laTailleImage; i++)
	{
		agrandit_pixel(i,telleImageDest,telleImageSource,telCoef);
	}
}

/* Agrandit le pixel indice de telCoef */
void agrandit_pixel(int indice,struct image* telleImageDest,
	      	    const struct image* telleImageSource,int telCoef)
{
	int laL,laC,j,k,lIndice;

	laL=indice/telleImageSource->saLargeur * telCoef;
	laC=indice%telleImageSource->saLargeur * telCoef;
		
	for(j=0; j<telCoef; j++)
	{
		for(k=0; k<telCoef; k++)
		{
			lIndice=(laL * telleImageDest->saLargeur + laC ) + 
				(j * telleImageDest->saLargeur) + k;

			telleImageDest->sonContenu[lIndice]=
					telleImageSource->sonContenu[indice];
		}
	}
}
