/* cacher_lib.c - Fonctions utiles pour cacher et decacher
 * ____________________________________________________________________
 *
 * Ces fonctions sont utilisees pour transformer un entier ou un
 * caractere en tableau de "bits" et inversement.
 * On peut donc remplacer aisement un ou plusieurs bits a l'aide de ces
 * tableaux et le retransformer en entier.
 * ____________________________________________________________________
 */

#include <stdlib.h>
#include <math.h>
#include "libimage.h"
#include "effets/cacher_lib.h"

/* Rempli un tableau de "bits" correspondant a un pixel
 * tableau de "bits" = un tableau ne contenant que des 0 et des 1
 */
char* rempli_tabRVB(int telComposanteRVB)
{
	char* leTabTmp;
	char* leTabRVB;
	int j;

	leTabRVB=(char*)ppm_malloc(3*8*sizeof(char));

	leTabTmp=get_tab_carac(get_red(telComposanteRVB));
	for(j=0; j<8; j++)
	{
		leTabRVB[j]=leTabTmp[j];
	}
	ppm_free((void*)leTabTmp);

	leTabTmp=get_tab_carac(get_green(telComposanteRVB));
	for(j=0; j<8; j++)
	{
		leTabRVB[8+j]=leTabTmp[j];
	}
	ppm_free((void*)leTabTmp);

	leTabTmp=get_tab_carac(get_blue(telComposanteRVB));
	for(j=0; j<8; j++)
	{
		leTabRVB[16+j]=leTabTmp[j];
	}
	ppm_free((void*)leTabTmp);

	return leTabRVB;
}

/* Renvoie un tableau de "bits" correspondant au
 * caractere passee en parametre
 */
char* get_tab_carac(int telC)
{
	char* leRetour;
	int i, cont;
	char leBitTmp;

	leRetour=(char*)ppm_malloc(8*sizeof(char));

	cont=0;
	for(i=7; i>=0; i--)
	{
		leBitTmp=(telC>>i)%2;
		leBitTmp=abs(leBitTmp);
		leRetour[cont]=leBitTmp;
		cont++;
	}

	return leRetour;
}

/* Retourne un entier correspondant au tableau de "bits" et au nombre de cases
 * passes en parametre.
 * 
 * Par exemple, si on passe l'adresse d'un char et l'entier 5, la fonction
 * retournera le nombre correspondant aux 5 bits suivants l'adresse.
 *		     ---------------------
 *	Si on passe: | 0 | 1 | 0 | 1 | 0 | et 5, la fonction retournera 10.
 *		     ---------------------
 */
int get_carac_from_tab_bits(const char* telTab, int telNbDeCases)
{
	int leRetour;
	int i;

	leRetour=0;
	for(i=0; i<telNbDeCases; i++)
	{
		leRetour+=telTab[i]*pow(2,telNbDeCases-1-i);
	}

	return leRetour;
}
