/* flou.c - Effectue differents flou sur image au format PPM
 * ____________________________________________________________________
 * Effectue un flou une image passee en parametre
 * Le programme accepte 3 parametres :
 *	- le premier doit etre un entier (coefficient de flou)
 * 	- le second est un type de flou 
 * 	- le dernier est le fichier ppm source
 * ____________________________________________________________________
 */

#include <stdio.h>
#include <string.h>
#include "io.h"
#include "progress.h"
#include "effets/flou.h"

int main(int argc, char** argv)
{
	int leRetour, leCoef;
	char* leFichierDest;
	struct image lImage;
	struct image lImageDest;
	char leSuffixeFichier[20]="_FLOU";

	if(argc>1 && strcmp(argv[1],"--help")==0)
	{
		usage(argv[0]);
		
		return NO_ERROR;	
	}

	if(argc!=4)
	{
		fprintf(stderr,"usage: %s [coef] [type] ",argv[0]);
		fprintf(stderr,"[img_source]\n");
		fprintf(stderr,"tapez %s --help ",argv[0]);
		fprintf(stderr,"pour plus de details\n");

		return ERROR_ARGS;
	}
	
	/* Le coefficient de flou */
	leCoef=atoi(argv[1]);	
	if(leCoef==0)
	{
		fprintf(stderr,"Le coefficient doit etre un entier.\n");
		usage(argv[0]);
		
		return ERROR_ARGS;		
	}
	
	/* Test de l'existence du flou passe en parametre */
	if(strcmp(argv[2],"-normal")!=0 && strcmp(argv[2],"-blur")!=0 &&
	   strcmp(argv[2],"-gauss")!=0 && strcmp(argv[2],"-motion")!=0 &&
	   strcmp(argv[2],"-radial")!=0 && strcmp(argv[2],"-spin")!=0)
	{
		fprintf(stderr,"Ce flou n'existe pas.\n");
		usage(argv[0]);
		
		return ERROR_ARGS;
	}
		
	/* Recupere l'image */
	if((leRetour=get_image(&lImage, argv[3])==NO_ERROR))
	{
		/* Applique l'effet */
		if(flou(&lImage,&lImageDest,leCoef,argv[2])==NO_ERROR)
		{
			/* Generation du nom de fichier */
			strcat(leSuffixeFichier,argv[2]);
			strcat(leSuffixeFichier,".ppm");
			leFichierDest=get_file_name(argv[3],leSuffixeFichier);
				
			/* Enregistre l'image */
			leRetour=create_image(&lImageDest, leFichierDest);
				
			/* Desallocation */
			ppm_free((void*)lImageDest.sonContenu);
			ppm_free((void*)leFichierDest);
		}
		else
		{
			fprintf(stderr,"Type de flou non reconnu\n");
			fprintf(stderr,"tapez %s --help ",argv[0]);
			fprintf(stderr,"pour plus de details\n");
		}

		/* Desallocation */
		ppm_free((void*)lImage.sonContenu);
	}

	return leRetour;
}

/* Appelle les differentes fonctions de flou */
int flou(const struct image* telleImageSource, struct image* telleImageDest,
	 int telCoef, char* telType)
{

	if(telCoef==0)
	{
		copie_image(telleImageDest, telleImageSource);
		return NO_ERROR;
	}


	printf("Application du flou  "); fflush(stdout);

	/* Flou normal */
	if(strcmp(telType,"-normal")==0)
	{
		copie_image(telleImageDest, telleImageSource);
		flou_normal(telleImageDest, telCoef);
	
		return NO_ERROR;
	}
	
	/* Flou optimise */
	if(strcmp(telType,"-blur")==0)
	{
		flou_optimise(telleImageDest,telleImageSource,telCoef);
		return NO_ERROR;
	}
	
	/* Flou gaussien */
	if(strcmp(telType,"-gauss")==0)
	{
		flou_gaussien(telleImageDest,telleImageSource,telCoef);
		return NO_ERROR;
	}

	/* Flou motion */
	if(strcmp(telType,"-motion")==0)
	{
		flou_motion(telleImageDest,telleImageSource,telCoef);
		return NO_ERROR;
	}

	/* Flou radial */
	if(strcmp(telType,"-radial")==0)
	{

		flou_radial(telleImageDest, telleImageSource,
			    telleImageSource->saLargeur>>1,
			    telleImageSource->saHauteur>>1, 100);

		
		
		return NO_ERROR;	
	}
	
	/* Flou circulaire */
	if(strcmp(telType,"-spin")==0)
	{
		/* Application du flou */ 
		spin_blur(telleImageDest, telleImageSource, telCoef);
		
		
		return NO_ERROR;	

	}

	return ERROR_ARGS;
}

void usage(char* telNom)
{
	fprintf(stderr,"usage: %s [coeff] [type] ",telNom);
	fprintf(stderr,"[image_source]\n");
	fprintf(stderr,"Le coeff doit etre compris entre 1 et 100.");
	fprintf(stderr,"\nType de flou : \n");
	fprintf(stderr,"\t-normal\t\tFlou normal\n");
	fprintf(stderr,"\t-blur\t\tFlou optimise\n");
	fprintf(stderr,"\t-gauss\t\tFlou gaussien\n");
	fprintf(stderr,"\t-motion\t\tFlou directionnel\n");	
	fprintf(stderr,"\t-radial\t\tFlou radial (pas de coef)\n");	
	fprintf(stderr,"\t-spin\t\tFlou ciculaire\n");	
}
