#include "libimage.h"
#include "progress.h"
#include "effets/flou.h"

void flou_gaussien(struct image* telleImageDest,
		   const struct image* telleImageSource,
		   int telCoef)
{
	int laTailleImage, leNbBoucle;
	int total, laL, i;
	struct image* lImageTmp;
	
	laTailleImage=telleImageSource->saHauteur*telleImageSource->saLargeur;
	leNbBoucle=laTailleImage*telCoef;
	total=0;

	lImageTmp=(struct image*)ppm_malloc(sizeof(struct image));

	/* Copie de l'image source dans l'image temporaire */
	copie_image(lImageTmp, telleImageSource);
		
	/* Copie de l'entete de l'image source dans
	 * l'image de destination
	 */
	copie_image_entete(telleImageDest,telleImageSource);
		
	/* Application du flou pour chaque pixel de l'image
	 * (telCoef fois)
	 */
	for(i=0;i<telCoef;i++)
	{
		for(laL=0;laL<laTailleImage;laL++)
		{
			calcul_pixel_gaussien(laL, lImageTmp,
					      telleImageDest);

			affiche_progress(total++, leNbBoucle);
		}
			
		ppm_free(lImageTmp->sonContenu);			
		copie_image(lImageTmp, telleImageDest);
	}
		
	termine_progress("OK");
	ppm_free(lImageTmp->sonContenu);
	

}

/* Flou gaussien */
void calcul_pixel_gaussien(int indice, const struct image* telleImageSource,
			   struct image* telleImageDest)
{
	/* Le flou gaussien a un principe similaire au flou normal.
	 * La difference est qu'on travaille sur un bloc de pixels plus grand
	 * (ici 5 * 5) et que chaque pixel de cette matrice aura plus ou moins
	 * d'importance selon son eloignement du pixel central (un pixel
	 * eloigne a moins d'impact sur le flou final qu'un pixel proche du
	 * pixel central. On attribue donc un poids a chaque pixel de la
	 * matrice 5 * 5 (appele plus loin valeur de ponderation).
	 * Le pixel du centre aura le poids le plus important (ici 41). Les
	 * valeurs de ponderation diminuent lorsqu'on s'eloigne du centre.
	 *
	 * L'algorithme fonctionne de maniere similaire au flou normal sauf
	 * lors de l'addition des pixels, il faut les multiplier par leurs
	 * valeurs de ponderation respectives (pris de la matrice). A la fin,
	 * on divise la valeur obtenue par la somme des valeurs de la matrice
	 * de ponderation (sauf les pixels ignores qui sortent de l'image).
	 * On ecrit le resultat dans une nouvelle structure.
	 */
	
	int totalR,totalV,totalB,tailleImage;
	int i,j,total_p,valeur,laValeur;
	int mat_p[5][5]={{1,4,7,4,1},{4,16,26,16,4},{7,26,41,26,7},
			{4,16,26,16,4},{1,4,7,4,1}};
	
	totalR=0;
	totalV=0;
	totalB=0;
	total_p=0;
	tailleImage=telleImageSource->saLargeur*telleImageSource->saHauteur;

	for(i=0;i<5;i++)
	{
		for (j=0;j<5;j++)
	      	{
	      		valeur=indice+(2-j)*telleImageSource->saLargeur+(i-2);
	      		if(valeur>=0 && valeur<tailleImage)
	      		{
				laValeur=telleImageSource->sonContenu[valeur];

	      			totalR+=mat_p[i][j]*get_red(laValeur);
	      			totalV+=mat_p[i][j]*get_green(laValeur);
	      			totalB+=mat_p[i][j]*get_blue(laValeur);

	      			total_p+=mat_p[i][j];
	      		}
	      	}
	}
	
	totalR /= total_p;
	totalV /= total_p;
	totalB /= total_p;	

	/* source != destination --> passer une structure vide */
	telleImageDest->sonContenu[indice]=get_code_hexa(totalR,totalV,totalB);
}
