#include "libimage.h"
#include "progress.h"
#include "effets/flou.h"

void flou_optimise(struct image* telleImageDest,
		   const struct image* telleImageSource, int telCoef)
{
	int laTailleImage;
	int leNbBoucle;
	int i, laL;
	int total;

	laTailleImage=telleImageSource->saLargeur*telleImageSource->saHauteur;
	leNbBoucle=laTailleImage*telCoef;
	total=0;

	copie_image(telleImageDest, telleImageSource);

	for(i=0;i<telCoef;i++)
	{
		for(laL=0;laL<laTailleImage;laL++)
		{
			calcul_pixel_blur(laL,telleImageDest);
			affiche_progress(total++,leNbBoucle);
		}
	}

	termine_progress("OK");
}

/* Flou optimise - prend seulement les pixels en diagonal du pixel central) */
void calcul_pixel_blur(int indice, struct image* telleImage)
{
	/* Le flou de type blur est une version optimise du flou normal.
	 * Il consiste a calculer la moyenne d'un groupe de pixels voisins.
	 * Nous allons detailler ici sa version optimise. Pour chaque pixel de
	 * l'image, on effectue l'addition des valeurs des pixels voisins que
	 * l'on divise a la fin par le nombre de pixels additionnes. On
	 * remplace alors le pixel courant par la nouvelle valeur. Dans notre
	 * cas, le bloc de pixels sera de taille 3*3. Le pixel central de
	 * cette matrice sera remplace a la fin du calcul. Pour accelerer les
	 * temps de calcul, nous prendrons selon les voisins situe en
	 * diagonale de ce pixel central.
	 *
	 * Pour accentuer le flou, nous avons opte pour l'option de faire
	 * plusieurs passes sur l'image.
	 */
	
	int totalR,totalV,totalB;
	int hautgauche,hautdroite,basgauche,basdroite,tailleImage;
	int i;

	i=0;
	totalR=0;
	totalV=0;
	totalB=0;
	tailleImage=telleImage->saLargeur*telleImage->saHauteur;

	hautgauche=indice-telleImage->saLargeur-1;
	hautdroite=indice-telleImage->saLargeur+1;
	basgauche=indice+telleImage->saLargeur-1;
	basdroite=indice+telleImage->saLargeur+1;

	if(hautgauche>=0 && hautgauche<tailleImage)
	{
		totalR += get_red(telleImage->sonContenu[hautgauche]);
		totalV += get_green(telleImage->sonContenu[hautgauche]);
		totalB += get_blue(telleImage->sonContenu[hautgauche]);
		i++;
	}

	if(hautdroite>=0 && hautdroite<tailleImage)
	{
		totalR += get_red(telleImage->sonContenu[hautdroite]);
		totalV += get_green(telleImage->sonContenu[hautdroite]);
		totalB += get_blue(telleImage->sonContenu[hautdroite]);
		i++;
	}

	if(basgauche<tailleImage)
	{
		totalR += get_red(telleImage->sonContenu[basgauche]);
		totalV += get_green(telleImage->sonContenu[basgauche]);
		totalB += get_blue(telleImage->sonContenu[basgauche]);
		i++;
	}

	if(basdroite<tailleImage)
	{
		totalR += get_red(telleImage->sonContenu[basdroite]);
		totalV += get_green(telleImage->sonContenu[basdroite]);
		totalB += get_blue(telleImage->sonContenu[basdroite]);
		i++;
	}

	totalR /= i;
	totalV /= i;
	totalB /= i;

	telleImage->sonContenu[indice]=get_code_hexa(totalR,totalV,totalB);
}
