#include "libimage.h"
#include "progress.h"
#include "effets/flou.h"

void flou_radial(struct image* telleImageDest,
		const struct image* telleImageSource,
		int cx, int cy, int nb_subdiv)
{
	/* Voir flou_radial_px pour les explications concernant le flou.
	 * Cette fonction permet d'appliquer le flou a tous les pixels de
	 * l'image. L'image source n'est pas modifiee, c'est l'image de
	 * destination qui contiendra le resultat du flou.
	 */
	
	int i, laTailleImage;
	
	/* Initialisation */
	laTailleImage=telleImageSource->saLargeur*telleImageSource->saHauteur;

	/* Copie de l'entete de l'image source dans
	 * l'image de destination
	 */
	copie_image_entete(telleImageDest,telleImageSource);
	
	/* On peut maintenant appliquer le flou sur tous les pixels
	 * de l'image source
	 */
	for(i=0; i<laTailleImage; i++)
	{
		flou_radial_px(i%telleImageSource->saLargeur,
			       i/telleImageSource->saLargeur,
			       cx, cy, nb_subdiv,
			       telleImageDest, telleImageSource);

		affiche_progress(i,laTailleImage);
	}
	
	termine_progress("OK");
}

void flou_radial_px(int px, int py, int cx, int cy, int nb_subdiv,
		    struct image* telleImageDest,
		    const struct image* telleImageSource)
{
	/* Pour ce flou, on va calculer pour chaque pixel la moyenne des pixels
	 * situes sur un vecteur de coordonnee (px-cx, py-cy).
	 * En effet, on trace entre un point central cx,cy un vecteur avec un
	 * pixel de cood px,py puis on subdivise ce vecteur en plusieurs sous
	 * vecteurs. Chaque sous vecteur possede alors a sa "pointe" un pixel.
	 * On fera la moyenne de tous les pixels contenu sur le vecteur
	 * principal. Cette fonction effectue le flou pour un seul pixel
	 * de coordonnee px,py.
	 */
	
	int vx, vy, startx, starty, tmpx, tmpy;
	int moyr, moyv, moyb;
	int i, nb, indice;

	vx = ((px-cx)<<16)/nb_subdiv;
	vy = ((py-cy)<<16)/nb_subdiv;

	startx = px<<16;  
	starty = py<<16;
	moyr  = 0;
	moyv  = 0;
	moyb  = 0;
	nb=0;

	for (i=0; i<nb_subdiv; i++)
	{
		tmpx=(i*vx+startx)>>16;
		tmpy=(i*vy+starty)>>16;
		
		if(tmpx<0)
			tmpx=0;
			
		if(tmpy<0)
			tmpy=0;
			
		if(tmpx>=telleImageSource->saLargeur)
			tmpx=telleImageSource->saLargeur-1;
		
		if(tmpy>=telleImageSource->saHauteur)
			tmpy=telleImageSource->saHauteur-1;
		
		indice=tmpy*telleImageSource->saLargeur+tmpx;
		moyr+= get_red(telleImageSource->sonContenu[indice]);
		moyv+= get_green(telleImageSource->sonContenu[indice]);
		moyb+= get_blue(telleImageSource->sonContenu[indice]);
		nb++;
	}

	moyr /= nb;
	moyv /= nb;
	moyb /= nb;
	telleImageDest->sonContenu[py*telleImageSource->saLargeur+px]=
			get_code_hexa(moyr, moyv, moyb);
}
