#include <stdlib.h>
#include <math.h>
#include "progress.h"
#include "error.h"
#include "effets/flou.h"

/* Tableau de correspondance entre coordonnees cartesiennes et polaires */
int* table_r_p;

/* Tableau de correspondance entre coordonnees polaires et cartesiennes */ 
int* table_p_r;

/* Spin blur */
void spin_blur(struct image* telleImageDest,
	     const struct image* telleImageSource, int telCoef)
{
	/* Il faut passer l'image en coordonnees polaires, appliquer un flou de
	 * base et repasser l'image en coordonnees cartesiennes.
	 */
	int i, j, laTailleImage,curProgress,curTotal;
	struct image* lImageTmp;

	/* Coefficient */
	if(telCoef==1)
	{
         	telCoef=2;
	}
		
	/* Initialisation */
	laTailleImage=telleImageSource->saLargeur*telleImageSource->saHauteur;

	curTotal=laTailleImage*3;
	curProgress=0;
	
	/* Initialisation des deux tableaux de correspondance */
	calcTabCartEtPol(telleImageSource->saLargeur,
			 telleImageSource->saHauteur);
	
	/* Copie des entetes */
	copie_image_entete(telleImageDest, telleImageSource);

	/* Copie des entetes de telleImageSource */
	lImageTmp=(struct image*)ppm_malloc(sizeof(struct image));
	copie_image_entete(lImageTmp, telleImageSource);
	
	
	/* Passage en coordonnees polaires
	 * On parcourt le tableau de correspondance table_r_p et on associe
	 * les donnees avec chaque indice du tableau
	 */
	for(i=0; i<laTailleImage; i++)
	{
		affiche_progress(curProgress++,curTotal);

		j=table_r_p[i];
		telleImageDest->sonContenu[i]=telleImageSource->sonContenu[j];
	}
	
	/* On applique un flou horizontal sur l'image
	 * polaires de coef telCoef
	 */
	for(j=0; j<laTailleImage; j++)
	{
		affiche_progress(curProgress++,curTotal);

		calcul_pixel_motion_bouclage(j, telCoef,
					    telleImageDest,
					    lImageTmp);
	}

	/* Passage de l'image en coordonnees cartesiennes
	 * Pour cela, on a besoin d'une image temporaire pour effectuer
	 * la correspondance avec lImageDest sans modifier l'image source.
	 */
	for(i=0; i<laTailleImage; i++)
	{
		affiche_progress(curProgress++,curTotal);

		j=table_p_r[i];
		telleImageDest->sonContenu[i]=lImageTmp->sonContenu[j];
	}
	
	termine_progress("OK");
	
	/* Desallocation */
	ppm_free((void*)table_r_p);
	ppm_free((void*)table_p_r);
	ppm_free((void*)lImageTmp->sonContenu);
	ppm_free((void*)lImageTmp);
	
	
}

int calcTabCartEtPol(int telleLargeur, int telleHauteur)
{
	int i, cx, cy, r, angle, x, y;
	float max_dist, real_r, real_angle;
	
	i=0;
	
	/* Initialisation des deux tableaux de correspondance */
	table_r_p=(int*)ppm_malloc(telleLargeur*telleHauteur*sizeof(int));
	table_p_r=(int*)ppm_malloc(telleLargeur*telleHauteur*sizeof(int));
	

	/* Initialisation du centre du flou 
	 * >>1 decalage d un bit vers la droite (division par 2)
	 * mais c'est plus rapide qu'une division.
	 */
	cx = telleLargeur>>1;
	cy = telleHauteur>>1;
	
	max_dist=sqrt(	(telleLargeur-cx)*(telleLargeur-cx)+
			(telleHauteur-cy)*(telleHauteur-cy)  );
	
	
	/* Premiere table (table_r_p) permet de passer des coordonnees
	 * cartesiennes (ou rectangulaire) en coordonnees polaires
	 *  (r,angle) -> (x,y)
	 */
	for(r=0; r<telleHauteur; r++)
	{
		for(angle=0; angle<telleLargeur; angle++)
		{
			/* On trouve a quel rayon dans l'image source
			 * correspond cette valeur
			 */
			real_r = (r*max_dist)/telleHauteur;
			
			/* Meme chose pour l'angle */
			real_angle = (angle*2.0f*3.1415f)/telleLargeur;
			
			/* On trouve le couple (x,y) qui correspond a ce
		 	 * (r, angle) */
			x = (int)(cx+real_r*sin(real_angle));
			y = (int)(cy+real_r*cos(real_angle));
			
			/* Mise a (0,0) si les valeurs sortent de l'image */
			if(x<0 || x>=telleLargeur || y<0 || y>=telleHauteur)
			{
				x=0;
				y=0;
			}
			
			/* Stockage dans le tableau */
			table_r_p[i++]=y*telleLargeur+x;
		}
	}
	
	/* Second tableau - permet de passer de coordonnees polaires en
	 * 		    coordonnees rectangulaires
	 */
	
	i=0;
	for(y=0; y<telleHauteur; y++)
	{
		for(x=0; x<telleLargeur; x++)
		{
			/* Distance entre le centre et le pixel courant */
			real_r = sqrt((double)((x-cx)*(x-cx)+(y-cy)*(y-cy)));
			
			/* Angle par rapport a l'axe des abscisses */
			real_angle = atan2(x-cx, y-cy);
			/* atan2(x,y)=arctan(x/y) */

			/* On change l'intervalle des valeurs pour rester
			 * dans les limites du tableau
			 */
			real_r*=telleHauteur/max_dist;
			real_angle*=telleLargeur/(2*3.1415f);
			
			/* On arrondit les valeurs */
			r=real_r;
			angle=real_angle;
			
			/* On controle pour etre sur que les valeurs sont bien
			 * dans l'intervalle 
			 */
			/*
			if(r<0 || r>=telleHauteur || angle<0 ||
				angle>=telleLargeur)
			{
				angle=0;
			}*/
			
			/* On stocke dans le tableau */
			table_p_r[i++]=angle+r*telleLargeur;
		}
	}
	

	return NO_ERROR;
}

/* Flou motion avec bouclage sur le debut de la ligne en cas de depassement
 * de coordonnees. Ne sert que pour le flou circulaire
 */
void calcul_pixel_motion_bouclage(int indice,int intensite,
				  const struct image* telleImageSrc,
				  struct image* telleImageDest)
{
	/* 
	 * Ce flou est different du motion conventionnel car quand un pixel
	 * sort de l'image, on revient en debut de ligne. En effet, si on est
	 * au pixel (x-5, y) sur une image de x pixels sur y pixels et une
	 * intensite de 10 on va jusqu'a (x,y) et au lieu d'ignorer les pixels
	 * suivants qui sont hors de l'image, on va boucler au debut de la
	 * ligne (cad de (0,y) a (5,y))
	 */
	
	int totalR,totalV,totalB,i,valeur,intensite_reel,tailleImage;
	
	totalR=0;
	totalV=0;
	totalB=0;
	intensite_reel=0;

	tailleImage=telleImageSrc->saLargeur*telleImageSrc->saHauteur;

   	for (i=-(intensite>>1);i<intensite>>1;i++)
   	{
   		valeur=indice+i;

		if(valeur>=telleImageSrc->saLargeur)
		{
			valeur-=telleImageSrc->saLargeur;
		}
		
		if(valeur<0)
		{
			valeur+=telleImageSrc->saLargeur;
		}
		
		
   		if(valeur>=0 && valeur<tailleImage)
   		{
      			totalR+=get_red(telleImageSrc->sonContenu[valeur]);
      			totalV+=get_green(telleImageSrc->sonContenu[valeur]);
      			totalB+=get_blue(telleImageSrc->sonContenu[valeur]);
      			intensite_reel++;
      		}
   	}

	totalR /= intensite_reel;
	totalV /= intensite_reel;
	totalB /= intensite_reel;

	telleImageDest->sonContenu[indice]=get_code_hexa(totalR,totalV,totalB);
}
