/*
 * miroir.c - Effectue un miroir de l'image horizontal ou vertical
 */

#include <stdio.h>
#include <string.h>
#include "io.h"
#include "progress.h"
#include "effets/miroir.h"

int main(int argc, char** argv)
{
	struct image lImageSource;
	struct image lImageDest;
	int leRetour;
	char* leFichierDest;
	char leSuffixeFichier[15]="_MIROIR";
	char leTypeMir;
	
	if(argc>1 && strcmp(argv[1],"--help")==0)
	{
		fprintf(stdout,"Usage: %s <type> <nom de l'image>\n",argv[0]);
		fprintf(stdout,"\ntype de miroir : \n");
		fprintf(stdout,"\t-h\t\tMiroir horizontal\n");
		fprintf(stdout,"\t-v\t\tMiroir vertical\n");

		return NO_ERROR;
	}
	
	if(argc!=3)
	{

		fprintf(stderr,"Usage: %s <type> <nom de l'image>\n",argv[0]);
		fprintf(stderr,"Tapez %s --help pour plus", argv[0]);
		fprintf(stderr," de details\n");

		return ERROR_ARGS;
	}
	
	leRetour=get_image(&lImageSource, argv[2]);
	if(leRetour==NO_ERROR)
	{
		leTypeMir=strcmp(argv[1], "-h")==0 ? HORIZONTAL : VERTICAL;
		
		if(leTypeMir==HORIZONTAL)
		{
			strcat(leSuffixeFichier, "_H.ppm");	
		}
		else
		{
			strcat(leSuffixeFichier, "_V.ppm");
		}

		/* Generation du nom de fichier de destination */
		leFichierDest=get_file_name(argv[2],leSuffixeFichier);

		/* Miroir */
		miroir(&lImageDest, &lImageSource, leTypeMir);
	
		/* Creation de l'image finale */
		leRetour=create_image(&lImageDest, leFichierDest);

		/* Desallocation de la memoire */
		free((void*)leFichierDest);
		free((void*)lImageSource.sonContenu);
		free((void*)lImageDest.sonContenu);
	}
	
	return leRetour;
}

void miroir(struct image* telleImageDest,
	    const struct image* telleImageSource, char h_ou_v)
{
	int i, j;
	int indDest,indSrc;
	int *leContenuSrc;

	leContenuSrc =telleImageSource->sonContenu;
	
	/* Copie des entetes et allocation */
	copie_image_entete(telleImageDest, telleImageSource);
	       
	switch(h_ou_v)
	{
		case HORIZONTAL:
		
		printf("Miroir horizontal    "); fflush(stdout);
			
		for(i=0; i<telleImageSource->saLargeur; i++)
		{
			affiche_progress(i,telleImageSource->saLargeur);
			
			for(j=0; j<telleImageSource->saHauteur; j++)
			{
				indDest=(telleImageSource->saHauteur-j-1)
					*telleImageSource->saLargeur+i;

				indSrc =j*telleImageSource->saLargeur+i;
				

				telleImageDest->sonContenu[indDest]=
							leContenuSrc[indSrc];
				
			}
		}
		break;
			
		case VERTICAL:
		default:
		
		printf("Miroir vertical      "); fflush(stdout);
		
		for(i=0; i<telleImageSource->saLargeur; i++)
		{
			affiche_progress(i,telleImageSource->saLargeur);
			
			for(j=0; j<telleImageSource->saHauteur; j++)
			{
				indDest=j*telleImageSource->saLargeur+
					(telleImageSource->saLargeur-i-1);

				indSrc =j*telleImageSource->saLargeur+i;

				telleImageDest->sonContenu[indDest]=
							leContenuSrc[indSrc];
			}
		}
		break;
	}	
	
	termine_progress("OK");
}
