/* negatif.c - Transforme toutes les couleurs d'une image en leur negatif
 * ____________________________________________________________________
 * Le negatif d'une couleur est donne par la formule suivante:
 * 	- Notons max, le maximum de couleurs de l'image.
 *	- Notons neg(x) la fonction qui donne le negatif d'une
 *	  composante x d'un pixel
 * 	- pour une composante RVB
 * 		->neg(R)=max-R
 * 		->neg(V)=max-V
 * 		->neg(B)=max-B
 *
 * 	-Nous executerons la manipulation en hexadecimal pour
 * 	 simplifier les calculs.
 * 	   ->Rappel: 0xRVB est obtenu en concatenant les valeurs
 * 	   hexadecimales de la composante R,V et B.
 * 	    0xmax est la concatenation de max en hexadecimal 3 fois
 *
 *	   	neg(0xRVB)=0xmax-0xRVB
 * ____________________________________________________________________
 */

#include <stdio.h>
#include <stdlib.h>
#include "io.h"
#include "progress.h"
#include "effets/negatif.h"

int main(int argc, char** argv)
{
	struct image lImage;
	char* leFichierDest;
	int leRetour;

	if(argc!=2)
	{
		fprintf(stderr,"usage: %s [image_source]\n",argv[0]);
		return ERROR_ARGS;
	}

	leRetour=get_image(&lImage, argv[1]);
	if(leRetour==NO_ERROR)
	{
		/* Generation du nom de fichier */
		leFichierDest=get_file_name(argv[1],"_NEG.ppm");

		/* Application de l'effet */
		negatif(&lImage);

		/* Ecriture de l'image */
		leRetour=create_image(&lImage, leFichierDest);

		/* Desallocation */
		ppm_free((void*)lImage.sonContenu);
		ppm_free((void*)leFichierDest);
	}
	
	return leRetour;
}

/* Met en negatif l'image passee en parametre */
void negatif(struct image * telleImg)
{
	int i;
	int leMaxColHex;
	int laTailleImage;

	printf("Negatif de l'image   "); fflush(stdout);

	/* Recupere le code hexadecimal de la couleur maximale */
	leMaxColHex=get_code_hexa(telleImg->sonMaxCol,
				  telleImg->sonMaxCol,
				  telleImg->sonMaxCol);
	

	/* On effectue le negatif de chaque pixel */
	i=0;
	laTailleImage=telleImg->saLargeur*telleImg->saHauteur;
	while(i<laTailleImage)
	{
		affiche_progress(i,laTailleImage);
		telleImg->sonContenu[i]=leMaxColHex-telleImg->sonContenu[i];
		i++;
	}

	termine_progress("OK");
}
