/* noiretblanc.c - Transforme une image PPM en niveaux de gris */

#include <stdio.h>
#include <stdlib.h>
#include "io.h"
#include "progress.h"
#include "effets/noiretblanc.h"

int main(int argc, char** argv)
{
	struct image lImage;
	char* leFichierDest;
	int leRetour;

	if(argc!=2)
	{
		fprintf(stderr,"usage: %s [image_source]\n",argv[0]);
		return ERROR_ARGS;
	}

	leRetour=get_image(&lImage, argv[1]);
	if(leRetour==NO_ERROR)
	{
		/* Generation du nom de fichier */
		leFichierDest=get_file_name(argv[1],"_NB.ppm");

		/* Application du filtre */
		noiretblanc(&lImage);

		/* Ecriture de l'image */
		leRetour=create_image(&lImage, leFichierDest);

		/* Desallocation */
		ppm_free((void*)lImage.sonContenu);
		ppm_free((void*)leFichierDest);
	}
	
	return leRetour;
}

/* Met en niveaux de gris l'image passee en parametre */
void noiretblanc(struct image * telleImage)
{
	int i;
	int laMoyenne, laTailleImage;

	printf("Application effet    "); fflush(stdout);

	/* Parcours de l'image */
	i=0;
	laTailleImage=telleImage->saLargeur*telleImage->saHauteur;
	while(i<laTailleImage)
	{
		affiche_progress(i,laTailleImage);

		/* On calcule la moyenne d'une composante RVB */
		laMoyenne=(get_red(telleImage->sonContenu[i])+
		     	   get_green(telleImage->sonContenu[i])+
		     	   get_blue(telleImage->sonContenu[i]))/3;

		/* On ecrit la valeur calcule dans chaque couleur
		 * d'une composante
		 */
		telleImage->sonContenu[i]=get_code_hexa(laMoyenne,
							laMoyenne,
							laMoyenne);
		i++;
	}

	termine_progress("OK");
}
