/* pixelize.c - Pixelize une image passee en parametre
 * ____________________________________________________________________
 * Le programme accepte 2 parametres, le premier doit etre un entier,
 * (representant un coefficient de pixelisation), le second est une
 * image au format ppm.
 * ____________________________________________________________________
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include "io.h"
#include "error.h"
#include "progress.h"
#include "effets/pixelize.h"

int main(int argc, char** argv)
{
	struct image lImage;
	char* leFichierDest;
	int leRetour, leCoef;

	leFichierDest=NULL;
	
	if(argc!=3)
	{
		fprintf(stderr,"usage: %s [coeff] [image_source]\n",argv[0]);
		return ERROR_ARGS;
	}

	/* Coefficient de pixelisation */
	leCoef=isdigit(*argv[1]) ? atoi(argv[1]): -1;
	
	if(leCoef==-1)
	{
		fprintf(stderr,"Le coefficient doit etre un entier.\n");
		fprintf(stderr,"usage: %s [coeff] [image_source]\n",argv[0]);
		return ERROR_ARGS;		
	}

	/* Recupere l'image */
	if((leRetour=get_image(&lImage, argv[2]))==NO_ERROR)
	{
		/* Applique l'effet */
		if((leRetour=pixelize(&lImage, leCoef))==NO_ERROR)
		{
			/* Generation du nom de fichier */
			leFichierDest=get_file_name(argv[2],"_pix.ppm");

			/* Enregistre l'image */
			leRetour=create_image(&lImage, leFichierDest);

			/* Desallocation */
			ppm_free((void*)lImage.sonContenu);
			ppm_free((void*)leFichierDest);

		}
	}

	return leRetour;
}

/* Pixelise l'image du coefficient passe en parametre */
int pixelize(struct image* telleImage, int telCoef)
{
	int a, b, laTailleCote;

	laTailleCote=2*telCoef+1;

	printf("Pixelisation         "); fflush(stdout);

	for(a=0; a<telleImage->saHauteur; a++)
	{
		affiche_progress(a,telleImage->saHauteur);

		for(b=0; b<telleImage->saLargeur; b++)
		{
			calcul_pixelize(telleImage,a,b,laTailleCote);
		}
	}

	termine_progress("OK");


	return NO_ERROR;
}

/* Pixelise un bloc de pixels */
void calcul_pixelize(struct image* telleImage,int a, int b,int telleTailleCote)
{
	int leCentreL, leCentreC;
	int laCouleur;
	int indCote;
	int indCourant;
	int curL, curC;
	int laL, laC;

	leCentreL=a*telleTailleCote;
	leCentreC=b*telleTailleCote;
	indCote=telleTailleCote/2;

	if(leCentreL<telleImage->saHauteur && leCentreC<telleImage->saLargeur)
	{
		indCourant=leCentreL*telleImage->saLargeur+leCentreC-1;
		laCouleur=telleImage->sonContenu[indCourant];

		for(laL=-indCote; laL<=indCote; laL++)
		{
			curL=leCentreL+laL;

			for(laC=-indCote; laC<=indCote; laC++)
			{
				curC=leCentreC+laC;

				if( curL<telleImage->saHauteur &&
				    curC<telleImage->saLargeur &&
				    curL>=0 &&  leCentreC+laC>=0
				  )
				{
					indCourant=curL*telleImage->saLargeur+
						   curC-1;
					telleImage->sonContenu[indCourant]=
								laCouleur;
				}
			}/*for laC*/
		}/*for laL*/
	}/*if*/

}
