/* Le 11/11/03 - Mathieu Lomon - Nicolas Maupu
 * rotation.c - Execute une rotation d'un angle passe en parametre
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "io.h"
#include "error.h"
#include "progress.h"
#include "effets/rotation.h"

#define DROITE 0
#define GAUCHE 1

int main(int argc, char** argv)
{
	struct image lImageSource;
	struct image lImageDest;
	char* leFichierDest;
	int leRetour, leCoef;
	char leSuffixeFichier[10]="_ROT";

	leFichierDest=NULL;
	leRetour=NO_ERROR;

	if(argc!=3 ||
	      (argc>=1 && strcmp(argv[1],"-d")!=0 && strcmp(argv[1],"-g")!=0))
	{
		fprintf(stderr,"usage: %s [sens] [image_source]\n",argv[0]);
		fprintf(stderr,"sens:\n\t-d\trotation a droite\n");
		fprintf(stderr,"\t-g\trotation a gauche\n");
		
		return ERROR_ARGS;
	}
	
	leCoef=strcmp(argv[1],"-d")==0 ? DROITE : GAUCHE;
		
	leRetour=get_image(&lImageSource, argv[2]);
	if(leRetour==NO_ERROR)
	{
		
		if(leCoef==DROITE)
		{
			strcat(leSuffixeFichier, "_D.ppm");
		}
		else
		{
			strcat(leSuffixeFichier, "_G.ppm");
		}
			
		/* Generation du nom de fichier de destination */
		leFichierDest=get_file_name(argv[2], leSuffixeFichier);

		/* Rotation */
		rotation(&lImageDest, &lImageSource, leCoef);

		/* Creation de l'image finale */
		leRetour=create_image(&lImageDest, leFichierDest);

		/* Desallocation de la memoire */
		ppm_free((void*)lImageSource.sonContenu);
		ppm_free((void*)lImageDest.sonContenu);
		ppm_free((void*)leFichierDest);
	}

	return leRetour;
}

void rotation(struct image* telleImgDest, 
	      const struct image* telleImgSrc, 
	      int telAngle)
{
	int i, j;
	
	#define LARG_S telleImgSrc->saLargeur
	#define LARG_D telleImgDest->saLargeur
	#define HAUT_S telleImgSrc->saHauteur
	#define HAUT_D telleImgDest->saHauteur
		
	/* Allocation de l'image de dedstination - on recopie l'image source
	 * en inversant la largeur avec la hauteur.
 	 */
	copie_image_entete_px(  telleImgDest, 
				telleImgSrc, 
				HAUT_S, 
				LARG_S);

	
	
	if(telAngle==DROITE)
	{
		/* Rotation a droite */
		printf("Rotation a droite    "); fflush(stdout);
				
		for(i=0; i<HAUT_S; i++)
		{
			affiche_progress(i,HAUT_S);
			for(j=0; j<LARG_S; j++)
			{
				/* le pixel source a pour coord (i,j)
				 * le pixel de destination aura pour coord:
				 * 	=>(j,(i-haut_img-1))
				 */
			      telleImgDest->sonContenu[j*LARG_D+(HAUT_S-i-1)]=
			      		telleImgSrc->sonContenu[i*LARG_S+j];
			}
		}
	}
	else
	{
		/* Rotation a gauche */
		printf("Rotation a gauche    "); fflush(stdout);
		
		for(i=0; i<HAUT_S; i++)
		{
			affiche_progress(i,HAUT_S);
			for(j=0; j<LARG_S; j++)
			{
				/* (i,j) => (larg_img-j-1, i) */
			      	telleImgDest->sonContenu[(LARG_S-j-1)*LARG_D+i]=
			      		telleImgSrc->sonContenu[i*LARG_S+j];
			}
		}

	}
	
	#undef LARG_S
	#undef LARG_D
	#undef HAUT_S
	#undef HAUT_D
	
	termine_progress("OK");
}
